#!/usr/bin/perl
use File::Path;
use File::Glob ':glob';
use File::Copy::Vigilant;
use Digest::MD5;
########## By Pazyryk. Use, alter and distribute as you please. ################
#
# Gets all files in specified folders (of specified extension) and copies into
# the "current" mod (the one with the largest version number) in the specified
# folders (creating folders and paths if necessary), without creating redundant
# items. It will add them to the .modinfo file with the correct md5 and
# "include=1". You can create as many source->destination folder sets as you
# need, each with its own specified file extension (User Settings below).
#
# WARNING! Script overwrites .modinfo file. Use at your own risk.
#
# Notes:
#  * When specifying source folders, files of non-specified extension are
#    ignored. (E.g., I keep .dds's in the same folder as original .jpeg's.) 
#  * If a file name already exists in .modinfo (between <File> and </File>)
#    then this script will skip that file, regardless of folder or include=1 or
#    =0. So you can call this script repetitively (e.g., by mistake or after
#    creating a new .dds file) without creating redundant items in your mod.
#  * The script uses the .modinfo file (and only that) to check for already
#    existing files. It will overwrite a file of the same name in the specified
#    destination folder if it isn't in .modinfo.
#
# Installing PERL and required moduals
#  * Free PERL download: http://www.activestate.com/activeperl/downloads.
#  * If your PERL build is like mine, then you will need to install two
#    moduals: Digest-MD5 and File-Copy-Vigilant. You can type "ppm" in any
#    command window to get the Perl Package Manager. If not installed, click
#    View->All Packages, then right-click->install on the needed modual. No,
#    you're not done yet. Look for the obscure green arrow (->) in the top
#    panel (mouse-over says "run marked actions") and press it. 

####################    User Settings    #######################################

$mod = "";		# e.g., "MyMod", not "MyMod (v 1)"
$mods_folder_path = "";	# Path to MODS folder; see examples below

$art_from[0] = "";	# Path to source files
$art_to[0] = "";	# Path relative to mod folder
$extension[0] = "dds";	# File type to copy/add (note: I've only tested dds)
			# Repeat last three lines with [1,2,...] as needed,
			# but they must be added in sets of three

# Notes:
# $mod must be without version!
# Example $mods_folder_path:
# "C:/Users/Me/Documents/My Games/Sid Meier's Civilization 5/MODS"
# (Note that spaces and ' are handled OK. Not sure about other weird chars.)
# Example art_from, art_to, extension triplet:
#	$art_from[0] = "C:/Users/Me/Desktop/MyArt/People";
#	$art_to[0] = "Art/HiRes2D/People";
#	$extension[0] = "dds";
#	Add more triplets as needed with index 1, 2, etc. 
#
# Paths can be relative if you like, so "" works as current dir (except
# $art_to[#] is always relative to mod folder).

# WARNINGS!
#  * The actual mod folder name must be of standard format "MyMod (v 21)" with
#    the same exact two spaces (the number can be any integer > 0).
#  * You should not have anything resembling " (v #)" in any of the User
#    Settings specified above!

################################################################################
$start_time = time();
# Stupid user stuff
if ($mod eq ''){
	die "Are you a blockhead? You must include a mod name in User Settings.
That means open the PERL script and change stuff.
Do you know how to do that?
While you're there, read the instructions.\n\n";
} elsif ($#art_from != $#art_to){
	die "You still haven't read the instructions, have you?
Your art_from and art_to thingies don't have an equal
number of indecies.\n\n";
} elsif ($mod =~ / \(v /){
	die "Baby Jesus! I told you not to put version number in
your mod name. Go back to User Settings and do it right.\n\n";
} else {
	$test_string = join ('', $mod, $mods_folder_path, @art_from, @art_to);
	if ($test_string =~/[\\\*]/){
		die "We don't need no stinking \\'s here. Change those to
\/'s. No \* or other funny stuff either. Characters like
's and 's might work but are definitely un-American";
	}
}

# Get path for named mod with the highest version number
$mod_globber = $mods_folder_path.'/'.$mod.' (v *)';
$mod_globber =~ s/^\///;
@mod_version_folders = glob($mod_globber);
foreach (@mod_version_folders){
	(/ \(v (\d+)\)/);
	if ($1 > $version){
		$version = $1;
		$mod_path = $_;
	}
}
if ($mod_path){
	print "\n********  Found mod folder  ********\n$mod_path\n";
} else {
	die "Can't find the mod folder";
}

# Get modinfo file and open for read/overwrite
$modfile_globber = $mod_path.'/*.modinfo';
@modinfo = glob($modfile_globber);
if ($#modinfo != 0){
	$number = $#modinfo + 1;
	print "Found $number *.modfile files in $mod_path\n";
	print join ("\n", @modinfo), "\n";
	die;
}
print "********  Found modinfo file  ********\n$modinfo[0]\n";

open MODINFO, "$modinfo[0]" or die "Could not open $modinfo[0] for reading";
#open TESTOUT, ">test.modinfo" or die "Could not open test.modinfo for writing";

# Save modinfo lines (file will be overwriten later) and store file names for redundancy checks
while(<MODINFO>){
	push @modinfo_lines, $_;
	if ($reading_file_section){
		if (/<\/Files>/){
			$reading_file_section = 0;
		} else {
			if (/\/([^\/]+)<\/File>/){
				push @existing_files, $1;
			}
		}
	} elsif (/<Files>/){
		$reading_file_section = 1;
	}
}

#print "Already existing files $#existing_files:\n";
#print join ("\n", @existing_files),"\n\n";

# Get all source file paths and file names; build destination paths
for ($i=0; $i<= $#art_from; $i++){
	$from_globber = $art_from[$i].'/*.'.$extension[$i];
	@files = glob($from_globber);
	$to = $mod_path.'/'.$art_to[$i];
	foreach (@files){
		/\/([^\/]+)$/;
		push @file_names, $1;
		push @source, $_;
		push @destination, $to.'/'.$1;
		push @mod_path_name, $art_to[$i].'/'.$1;
	}
}
$number = $#file_names + 1;
if ($number < 1){
	print "Did not find any source files. Last glob string used:\n$from_globber\n";
	die;
} else {
	print "\n********  Found $number source files  ********\n";
	print join ("\n", @source),"\n";
}

# Test for redundancy in source file names (die before any writing)
for ($i=0; $i<= $#file_names; $i++){
	for ($j=$i+1; $j<= $#file_names; $j++){
		if ($file_names[$i] eq $file_names[$j]){
			push @redundancies, $source[$i], $source[$j];
		}
	}
}
if ($#redundancies >= 0){
	print "\nYou have redundant file names in your source folders:\n";
	print join ("\n", @redundancies),"\n\n";
	print "Stopping before any file copying or modinfo overwriting...";
	die;
}
# Create any needed directories
print "\n********  Creating needed mod folders  ********\n";
for ($i=0; $i<= $#art_from; $i++){
	mkpath($mod_path.'/'.$art_to[$i], { verbose => 1}) || print $!;
}

# Start overwriting modfile with saved modfile lines (start read/md5/copy loop after <File> line)
print "\n*********  Adding files to mod folders and modinfo  **********\n";
open MODINFO, ">$modinfo[0]" or die "Could not open $modinfo[0] for writing";
foreach (@modinfo_lines){
	print MODINFO $_;
	next unless (/<Files>/);	# only once after <Files> line
	$copied = 0;
	for ($i = 0; $i <= $#source; $i++){
		$redundant = 0;
		foreach (@existing_files){
			if ($file_names[$i] eq $_){
				$redundant = 1;
				last;
			}
		}
		if ($redundant == 1){
			print "$file_names[$i] already listed in modinfo, skipping...\n";
			next;
		}
		# md5 code that I don't understand
		open(FILE, $source[$i]) or die "Can't open $source[$i]: $!";
		binmode(FILE);
		$md5 = Digest::MD5->new->addfile(*FILE)->hexdigest;
		$md5 =~ tr/abcdef/ABCDEF/;
		# now copy files
		# This function uses md5 as validation but doesn't give it to you.
		# So md5 has been generated 3 times for every file. Someone could optimize
		# by writing own copy function, if they offended.
		if (copy_vigilant($source[$i], $destination[$i])){
			# Success! Now add to modinfo
			print MODINFO "    <File md5=\"$md5\" import=\"1\">$mod_path_name[$i]<\/File>\n";
			print "$mod_path_name[$i] \| $md5\n";
			$copied++;
		} else {
			print "!!!!Copy failure for $file_names[$i]: $!\n\tContinuing without adding this one to modinfo...\n";
		}
	}
}
$time = time() - $start_time;
print "\n$copied files added in $time seconds\n\n";


# So ugly...   If Lua is an elf, then PERL is an orc.

